from __future__ import absolute_import
__author__ = 'tarzan'

import importlib
from sqlalchemy import Column, types, Table, MetaData
from sqlalchemy.orm import mapper
from datetime import datetime

Base = None
DBSession = None

meta = MetaData()


def setup_database(session):
    global DBSession
    DBSession = session


_user_table = Table("rest_user", meta,
                    Column("id", types.Integer, primary_key=True,
                           autoincrement=True),
                    Column("username", types.VARCHAR(length=64),
                           unique=True, nullable=False),
                    Column("password", types.VARCHAR(length=64),
                           nullable=False),
                    Column("desc", types.TEXT),
                    Column("groups", types.TEXT),
                    Column("enabled", types.BOOLEAN,
                           default=True),
                    Column("last_modified_time", types.DateTime,
                           nullable=False,
                           default=datetime.now, onupdate=datetime.now)
)


class RESTfulUser(object):
    def __init__(self, **kwargs):
        self.id = None
        self.username = None
        self.password = None
        self.desc = None
        self.groups = None
        self.enabled = None
        self.last_modified_time = None
        self.__dict__.update(kwargs)

    def __unicode__(self):
        return self.username

    def __str__(self):
        return self.__unicode__().encode('utf-8')


mapper(RESTfulUser, _user_table)


def get_rest_user(username):
    user = DBSession.query(RESTfulUser).filter(
        RESTfulUser.username == username,
        RESTfulUser.enabled).first()
    if user is not None:
        return {
            'id': user.id,
            'username': user.username,
            'password': user.password,
            'roles': user.groups,
        }
    return None


def includeme(config):
    """
    :type config: pyramid.config.Configurator
    """
    dbsession_path = config.registry.settings["pyramid_restful_toolkit.auth.dbsession"]
    module_name, attr_name = dbsession_path.rsplit('.', 1)
    module = importlib.import_module(module_name, package=None)
    dbsession = getattr(module, attr_name)
    setup_database(dbsession)

    config.registry.settings['pyramid_restful_toolkit.auth.get_user_callback'] = \
        'pyramid_restful_toolkit.auth.models.sqlalchemy.get_rest_user'


"""Try to automatically declare backend schema
if pyramid_backend has installed"""
try:
    import pyramid_backend
    import colander
    from deform import widget

    class RESTfulUserSchema(colander.MappingSchema):
        username = colander.SchemaNode(colander.String())
        password = colander.SchemaNode(colander.String())
        enabled = colander.SchemaNode(colander.Boolean())
        groups = colander.SchemaNode(colander.String(), missing=colander.null)
        desc = colander.SchemaNode(colander.String(), missing=colander.null)

    RESTfulUser.__backend_schema_cls__ = RESTfulUserSchema

except ImportError:
        pass



__author__ = 'tarzan'


def jsonize_uncaught_exception_tween_factory(handler, registry):
    """
    This tween prevent all uncaught exception and return JSON response with error code 500
    """
    def jsonize_uncaught_exception_tween(request):
        """
        :type request: pyramid.request.Request
        :rtype : pyramid.response.Response
        """
        try:
            return handler(request)
        except BaseException, e:
            import json
            from pyramid import response

            body = json.dumps({
                'error': e.__class__.__name__,
                'code': 500,
                'status': '500 Internal Server Error',
                'message': e.message,
            })
            return response.Response(
                body=body,
                status='500 Internal Server Error',
                content_type='application/json',
            )

    return jsonize_uncaught_exception_tween


__author__ = 'tarzan'

import re

from pyramid.response import Response
from pyramid.security import Authenticated, Everyone
from .adapters import BaseAdapter
from .adapters.http_basic import HttpBasicAdapter
from .adapters.http_digest import HttpDigestAdapter


class AuthPolicy(object):
    """
    An authentication proxy for HTTP Header Auth (Basic, Digest  ...).

    This auto performs by parsing the header (request.authorization) then
    use a callback to fetch User information, after that it will do the
    authentication corresponding to fetched user

    User information:
    user = {
        "password": "the plain password"
        "roles": "the roles that are concatenated by ; or ,"
    }

    Callback function:
    def get_user(id):
        # id maybe username (It is, for Http Basic and Http Digest)
        pass
    """

    adapters = {
        'dummy': BaseAdapter,
        'basic': HttpBasicAdapter,
        'digest': HttpDigestAdapter,
    }

    def __init__(self, get_user_callback):
        self.get_user_callback = get_user_callback

    def _create_adapter(self, name, request):
        return self.adapters[name](request, self.get_user_callback_wrapper)

    def _get_adapter(self, request):
        """
        Get adapter to process authentication based on request
        """
        try:
            return request.__rest_auth_adapter__
        except AttributeError:
            try:
                method, args = request.authorization
                adapter_name = method.lower()
            except (ValueError, AttributeError, TypeError):
                adapter_name = 'dummy'
            request.__rest_auth_adapter__ = \
                self._create_adapter(adapter_name, request)
        return request.__rest_auth_adapter__

    def get_user_callback_wrapper(self, user_id):
        if user_id is None:
            return None
        else:
            _u = self.get_user_callback(user_id)
            if not isinstance(_u, dict):
                u = {
                    'username': None,
                    'password': None,
                    'roles': '',
                }
                for k in u:
                    try:
                        u[k] = getattr(_u, k)
                    except AttributeError:
                        pass
            else:
                u = _u
        if not u['roles']:
            u['roles'] = ''
        if not isinstance(u['roles'], str):
            u['roles'] = u['roles'].encode('utf-8')
        return u

    def _get_current_user(self, request):
        """
        Get User info for current request

        :return User info
        """
        try:
            return request.__rest_auth_current_user__
        except AttributeError:
            user_id = self.unauthenticated_userid(request)
            request.__rest_auth_current_user__ = \
                self.get_user_callback_wrapper(user_id)
        return request.__rest_auth_current_user__

    def unauthenticated_userid(self, request):
        return self._get_adapter(request).unauthenticated_userid()

    def authenticated_userid(self, request):
        return self._get_adapter(request).authenticated_userid()

    def effective_principals(self, request):
        user_id = self.authenticated_userid(request)
        if user_id is None:
            return []
        user = self._get_current_user(request)
        roles = [Everyone, Authenticated] + re.split(r'[;,]', user['roles'])
        return roles

    def remember(self, request, principal, **kw):
        return ()

    def forget(self, request):
        return ()

    def login_required(self, request):
        adapter = self._get_adapter(request)
        if not hasattr(adapter, 'login_required'):
            adapter = self._create_adapter('digest', request)
        return adapter.login_required(request)

    def forbidden(self, request):
        if not self.authenticated_userid(request):
            return self.login_required(request)

        r = Response()
        r.status_code = 403
        r.content_type = 'application/json'
        r.body = '{"errors":{"code":403,"message":"You are not authorized to access this API"}}'
        return r

    def _fetch_user_from_request(self, request):
        return None

    def get_user(self, request):
        try:
            return request.__rest_user__
        except AttributeError:
            request.__rest_user__ = self._fetch_user_from_request(request)
        return request.__rest_user__


__author__ = 'tarzan'

from datetime import datetime
import pyramid.renderers


def create_json_renderer():
    """
    Create a JSON renderer most of common datatype
    :rtype : pyramid.renderers.JSON

    >>> from datetime import datetime
    >>> renderer = create_json_renderer()
    >>> renderer = renderer(None)
    >>> renderer('abc', {})
    '"abc"'
    >>> renderer({1, 2, 3, 4}, {})
    '[1, 2, 3, 4]'
    >>> renderer(datetime.utcfromtimestamp(0), {})
    '"1970-01-01T00:00:00"'
    >>> try:
    ...     from cqlengine.models import Model
    ...     from cqlengine import columns
    ...     class TestModel(Model):
    ...         id = columns.Integer(primary_key=True)
    ...         set_col = columns.Set(columns.Integer)
    ...     m = TestModel(id=1234, set_col={1, 2, 3, 4})
    ... except ImportError:
    ...     class TestModel(dict):
    ...         def _as_dict(self):
    ...             return self
    ...     m = TestModel(id=1234, set_col={1, 2, 3, 4})
    >>>
    >>> renderer(m._as_dict(), {})
    '{"set_col": [1, 2, 3, 4], "id": 1234}'
    >>> renderer(m, {})
    '{"set_col": [1, 2, 3, 4], "id": 1234}'
    """
    r = pyramid.renderers.JSON()

    r.add_adapter(set, lambda obj, request: list(obj))
    r.add_adapter(datetime, lambda obj, request: obj.isoformat())

    try:
        import cqlengine.columns
        import cqlengine.models
        r.add_adapter(cqlengine.models.Model,
                      lambda obj, request: obj._as_dict())
        r.add_adapter(cqlengine.columns.ValueQuoter,
                      lambda obj, request: obj.value)
    except ImportError:
        pass

    return r


default_renderer = create_json_renderer()


__author__ = 'tarzan'

import importlib

from pyramid_restful_toolkit.auth.auth_policy import AuthPolicy


def includeme(config):
    """
    :type config: pyramid.config.Configurator
    """
    get_user_callback_path = config.registry.settings[
        "pyramid_restful_toolkit.auth.get_user_callback"]
    module_name, attr_name = get_user_callback_path.rsplit('.', 1)
    module = importlib.import_module(module_name, package=None)
    get_user_callback = getattr(module, attr_name)

    auth = AuthPolicy(get_user_callback=get_user_callback)
    config.set_authentication_policy(auth)
    config.add_forbidden_view(auth.forbidden)



__author__ = 'tarzan'

from . import BaseAdapter
import base64

class HttpBasicAdapter(BaseAdapter):

    def _parse_authorization_token(self):
        try:
            if not self.request.authorization:
                return None
            method, args = self.request.authorization
            _token = base64.b64decode(args)
            self._username, self._password = _token.split(':', 1)
        except (ValueError, TypeError, AttributeError), e:
            self._username = None
            self._password = None

    @property
    def username(self):
        try:
            return self._username
        except AttributeError:
            self._parse_authorization_token()
        return self._username

    @property
    def password(self):
        try:
            return self._password
        except AttributeError:
            self._parse_authorization_token()
        return self._password

    def unauthenticated_userid(self):
        return self.username

    def authenticated_userid(self):
        if not self.username:
            return None

        user = self.get_user_callback(self.username)
        if user['password'] == self.password:
            return self.username
        return None



__author__ = 'tarzan'

import json
import unittest
from pyramid.config import Configurator
from webtest import TestApp
import logging
import pytest
import sys

logging.basicConfig(level=logging.INFO)

try:
    import schema
except ImportError:
    pass

try:
    import colander
    import deform
except ImportError:
    pass

try:
    import formencode
    from formencode import validators
except ImportError:
    pass


def sample_app(app_settings={}):

    def colander_view(request):
        class TestSchema(colander.MappingSchema):
            a = colander.SchemaNode(colander.Integer())
        s = TestSchema()
        return s.deserialize(request.POST)

    def schema_view(request):
        s = schema.Schema({
            'a': int,
        })
        return s.validate(request.POST.mixed())


    def deform_view(request):
        class TestSchema(colander.MappingSchema):
            a = colander.SchemaNode(colander.Integer())
        form = deform.Form(schema=TestSchema())
        return form.validate(request.POST.items())

    def formencode_view(request):
        class TestSchema(formencode.Schema):
            allow_extra_fields = True
            a = validators.Int()

        s = TestSchema()
        return s.to_python(request.POST)

    def error_response_view(request):
        from pyramid_restful_toolkit import ErrorResponse
        raise ErrorResponse(407, 'ha ha ha', 'noway')

    settings = {}
    settings.update(app_settings)
    config = Configurator(
        settings=settings,
    )
    config.include('pyramid_restful_toolkit')

    config.add_route('colander', '/colander')
    config.add_view(colander_view, route_name='colander', renderer='json')

    config.add_route('schema', '/schema')
    config.add_view(schema_view, route_name='schema', renderer='json')

    config.add_route('deform', '/deform')
    config.add_view(deform_view, route_name='deform', renderer='json')

    config.add_route('formencode', '/formencode')
    config.add_view(formencode_view, route_name='formencode', renderer='json')

    config.add_route('error_response', '/error_response')
    config.add_view(error_response_view, route_name='error_response', renderer='json')

    app = config.make_wsgi_app()
    return TestApp(app)


class ErrorHandlersTest(unittest.TestCase):
    def setUp(self):
        super(ErrorHandlersTest, self).setUp()
        self.app = sample_app({})

    def test_all_schemas(self):
        data = {'a': 'ha ha ha',
                'b': 2}
        for api_path in ['colander', 'schema', 'deform', 'formencode']:
            if api_path not in sys.modules: continue;

            rs = self.app.post('/'+api_path, params=data, expect_errors=True)
            """:type : webtest.TestResponse"""
            self.assertEqual(rs.status_int, 400)
            rdata = json.loads(rs.text)
            self.assertIn('error_code', rdata)
            self.assertEqual(rdata['error_code'], 400)
            self.assertIn('errors', rdata)
            self.assertTrue(len(rdata['errors']))

    def test_error_response(self):
        rs = self.app.get('/error_response', expect_errors=True)
        """:type : webtest.TestResponse"""
        self.assertEqual(rs.status_int, 407)
        rdata = json.loads(rs.text)

        self.assertIn('error_code', rdata)
        self.assertEqual(rdata['error_code'], 407)

        self.assertIn('errors', rdata)
        self.assertEqual(rdata['errors'], 'ha ha ha')

        self.assertIn('data', rdata)
        self.assertEqual(rdata['data'], 'noway')



__author__ = 'tarzan'

import time
import os
import hashlib
import re
import unittest
from pyramid import testing
from pyramid.config import Configurator
from pyramid.security import Everyone, Authenticated
from pyramid_restful_toolkit.auth import AuthPolicy
from webtest import TestApp
from webob.descriptors import parse_auth
import logging

logging.basicConfig(level=logging.INFO)


def sample_app(app_settings={}, get_user_callback=None):
    def home_view(request):
        return 'he he'

    auth_policy = AuthPolicy(get_user_callback)
    settings = {
        'a': 'b',
    }
    settings.update(app_settings)
    config = Configurator(
        settings=settings,
        authentication_policy=auth_policy,
    )
    config.add_forbidden_view(auth_policy.forbidden)

    config.add_route('home', '/')

    config.add_view(home_view,
                    route_name='home',
                    renderer='string',
                    permission='view')

    app = config.make_wsgi_app()
    return TestApp(app)


class AuthPolicyTest(unittest.TestCase):
    def setUp(self):
        super(AuthPolicyTest, self).setUp()
        self.auth_policy = AuthPolicy(self.get_user_function)
        self.app = sample_app({}, self.get_user_function)

    def _create_request(self):
        return testing.DummyRequest(
            path='/path/to/api',
            params={
                'arg1': 'arg1 value',
                'arg2': 'arg2 value',
            },
        )

    def get_user_function(self, user_id):
        _users = {
            'gaia': {
                'password': 'lion',
                'roles': 'gaia,titan'
            },
            'Mufasa': {
                'password': 'Circle Of Life',
                'roles': 'vice-lionking',
            }
        }
        try:
            return _users[user_id]
        except KeyError:
            return None

    def test_dummy(self):
        request = self._create_request()
        ap = self.auth_policy

        self.assertIsNone(ap.unauthenticated_userid(request))
        self.assertIsNone(ap.authenticated_userid(request))
        self.assertListEqual(ap.effective_principals(request), [])

    def test_basic(self):
        ap = self.auth_policy

        # test with exist user, correct password
        request = self._create_request()
        request.authorization = ('Basic', 'Z2FpYTpsaW9u')  # gaia:lion
        self.assertEqual(ap.unauthenticated_userid(request), 'gaia')
        self.assertEqual(ap.authenticated_userid(request), 'gaia')
        self.assertSetEqual(
            set(ap.effective_principals(request)),
            set(('titan', 'gaia', Everyone, Authenticated))
        )

        # test with exist user, incorrect password
        request = self._create_request()
        request.authorization = ('Basic', 'Z2FpYTpsaW9uMQ==')  # gaia:lion1
        self.assertEqual(ap.unauthenticated_userid(request), 'gaia')
        self.assertIsNone(ap.authenticated_userid(request))
        self.assertListEqual(ap.effective_principals(request), [])

        # test with non-exist user
        request = self._create_request()
        request.authorization = ('Basic', 'emV1czpwaGFudG9t')  # zeus:phantom
        self.assertEqual(ap.unauthenticated_userid(request), 'zeus')
        self.assertIsNone(ap.authenticated_userid(request))
        self.assertListEqual(ap.effective_principals(request), [])

    def test_digest(self):
        # From mitsuhiko/werkzeug (used with permission).
        def unquote_header_value(value, is_filename=False):
            r"""Unquotes a header value.  (Reversal of :func:`quote_header_value`).
            This does not use the real unquoting but what browsers are actually
            using for quoting.

            :param value: the header value to unquote.
            """
            if value and value[0] == value[-1] == '"':
                # this is not the real unquoting, but fixing this so that the
                # RFC is met will result in bugs with internet explorer and
                # probably some other browsers as well.  IE for example is
                # uploading files with "C:\foo\bar.txt" as filename
                value = value[1:-1]

                # if this is a filename and the starting characters look like
                # a UNC path, then just return the value without quotes.  Using the
                # replace sequence below on a UNC path has the effect of turning
                # the leading double slash into a single slash and then
                # _fix_ie_filename() doesn't work correctly.  See #458.
                if not is_filename or value[:2] != '\\\\':
                    return value.replace('\\\\', '\\').replace('\\"', '"')
            return value

        # From mitsuhiko/werkzeug (used with permission).
        def parse_dict_header(value):
            """Parse lists of key, value pairs as described by RFC 2068 Section 2 and
            convert them into a python dict:

            >>> d = parse_dict_header('foo="is a fish", bar="as well"')
            >>> type(d) is dict
            True
            >>> sorted(d.items())
            [('bar', 'as well'), ('foo', 'is a fish')]

            If there is no value for a key it will be `None`:

            >>> parse_dict_header('key_without_value')
            {'key_without_value': None}

            To create a header from the :class:`dict` again, use the
            :func:`dump_header` function.

            :param value: a string with a dict header.
            :return: :class:`dict`
            """
            from urllib2 import parse_http_list as _parse_list_header

            result = {}
            for item in _parse_list_header(value):
                if '=' not in item:
                    result[item] = None
                    continue
                name, value = item.split('=', 1)
                if value[:1] == value[-1:] == '"':
                    value = unquote_header_value(value[1:-1])
                result[name] = value
            return result  # test unauthorized

        def build_digest_header(method, url, chals, username, password):
            from urlparse import urlparse

            realm = chals['realm']
            nonce = chals['nonce']
            qop = chals.get('qop')
            algorithm = chals.get('algorithm')
            opaque = chals.get('opaque')

            if algorithm is None:
                _algorithm = 'MD5'
            else:
                _algorithm = algorithm.upper()
            # lambdas assume digest modules are imported at the top level
            if _algorithm == 'MD5':
                def md5_utf8(x):
                    if isinstance(x, str):
                        x = x.encode('utf-8')
                    return hashlib.md5(x).hexdigest()

                hash_utf8 = md5_utf8
            elif _algorithm == 'SHA':
                def sha_utf8(x):
                    if isinstance(x, str):
                        x = x.encode('utf-8')
                    return hashlib.sha1(x).hexdigest()

                hash_utf8 = sha_utf8
            # XXX MD5-sess
            KD = lambda s, d: hash_utf8("%s:%s" % (s, d))

            if hash_utf8 is None:
                return None

            # XXX not implemented yet
            entdig = None
            p_parsed = urlparse(url)
            path = p_parsed.path
            if p_parsed.query:
                path += '?' + p_parsed.query

            A1 = '%s:%s:%s' % (username, realm, password)
            A2 = '%s:%s' % (method, path)

            if qop is None:
                respdig = KD(hash_utf8(A1), "%s:%s" % (nonce, hash_utf8(A2)))
            elif qop == 'auth' or 'auth' in qop.split(','):
                import random

                nonce_count = random.randint(1, 1000)
                ncvalue = '%08x' % nonce_count
                s = str(nonce_count).encode('utf-8')
                s += nonce.encode('utf-8')
                s += time.ctime().encode('utf-8')
                s += os.urandom(8)

                cnonce = (hashlib.sha1(s).hexdigest()[:16])
                noncebit = "%s:%s:%s:%s:%s" % (nonce, ncvalue, cnonce, qop, hash_utf8(A2))
                respdig = KD(hash_utf8(A1), noncebit)
            else:
                # XXX handle auth-int.
                return None

            # XXX should the partial digests be encoded too?
            base = 'username="%s", realm="%s", nonce="%s", uri="%s", ' \
                   'response="%s"' % (username, realm, nonce, path, respdig)
            if opaque:
                base += ', opaque="%s"' % opaque
            if algorithm:
                base += ', algorithm="%s"' % algorithm
            if entdig:
                base += ', digest="%s"' % entdig
            if qop:
                base += ', qop=auth, nc=%s, cnonce="%s"' % (ncvalue, cnonce)

            return 'Digest %s' % (base)

        # test unauthorized
        request = self._create_request()
        ap = self.auth_policy
        self.assertIsNone(ap.unauthenticated_userid(request))
        self.assertIsNone(ap.authenticated_userid(request))
        self.assertSetEqual(
            set(ap.effective_principals(request)),
            set([]))


        # test incorrect password
        r = ap.login_required(request)
        self.assertEqual(r.status_int, 401)
        self.assertIn('www-authenticate', r.headers)
        www_authenticate = r.headers['www-authenticate']
        print www_authenticate
        method, s_chals = re.split(r'\s+', www_authenticate, 1)
        self.assertEqual(method.lower(), 'digest')
        chals = parse_dict_header(s_chals)
        authorization = build_digest_header('GET', '/', chals, 'gaia', 'lion1')
        request = self._create_request()
        request.authorization = parse_auth(authorization)
        ap = self.auth_policy
        self.assertEqual(ap.unauthenticated_userid(request), 'gaia')
        self.assertIsNone(ap.authenticated_userid(request))
        self.assertSetEqual(
            set(ap.effective_principals(request)),
            set([]))

        # # test correct password
        r = ap.login_required(request)
        self.assertEqual(r.status_int, 401)
        self.assertIn('www-authenticate', r.headers)
        www_authenticate = r.headers['www-authenticate']
        method, s_chals = re.split(r'\s+', www_authenticate, 1)
        self.assertEqual(method.lower(), 'digest')
        chals = parse_dict_header(s_chals)
        authorization = build_digest_header('GET', '/', chals, 'gaia', 'lion')
        request = self._create_request()
        request.authorization = parse_auth(authorization)
        ap = self.auth_policy
        self.assertEqual(ap.unauthenticated_userid(request), 'gaia')
        self.assertEqual(ap.authenticated_userid(request), 'gaia')
        self.assertSetEqual(
            set(ap.effective_principals(request)),
            set(['gaia', 'titan', Everyone, Authenticated]))



__author__ = 'tarzan'
from datetime import datetime


class ErrorResponse(Exception):
    def __init__(self, code, errors=None, data=None):
        self.code = code
        self.errors = errors or {}
        self.data = data or {}
        self.message = '%d: %s %s' % (code, errors, data)

    def response(self, request):
        request.response.status_code = self.code
        data = {'data': self.data} if self.data else {}
        data['error_code'] = self.code
        data['errors'] = self.errors
        return data


from .tweens import jsonize_uncaught_exception_tween_factory
from .renderer import default_renderer


def includeme(config):
    """
    :type config: pyramid.config.Configurator
    """
    config.add_renderer(None, default_renderer)
    config.add_renderer('json', default_renderer)
    config.include(__name__ + '.error_handlers')


__author__ = 'tarzan'

from . import ErrorResponse


def _make_error_response(status_code, errors=None, data=None):
    data = {'data': data} if data else {}
    data['error_code'] = status_code
    data['errors'] = errors or {}
    return data


def on_schema_error(context, request):
    """
    :type context: _SchemaError
    :type request: pyramid.request.Request
    """
    request.response.status_code = 400
    return _make_error_response(400, context.autos)


def on_colander_invalid(context, request):
    """
    :type context: colander.Invalid
    :type request: pyramid.request.Request
    """
    request.response.status_code = 400
    return _make_error_response(400, context.asdict())


def on_formencode_invalid(context, request):
    """
    :type context: formencode.Invalid
    :type request: pyramid.request.Request
    """
    request.response.status_code = 400
    return _make_error_response(400, context.unpack_errors())


def on_error_response(context, request):
    """
    :type context: ErrorResponse
    """
    return context.response(request)


def on_deform_validation_failure(context, request):
    """
    :type context: deform.ValidationFailure
    """
    request.response.status_code = 400
    return _make_error_response(400, context.error.asdict())


def includeme(config):
    """
    :type config: pyramid.config.Configurator
    """
    try:
        import formencode
        config.add_view(on_formencode_invalid, context=formencode.Invalid)
    except ImportError:
        pass

    try:
        import colander
        config.add_view(on_colander_invalid, context=colander.Invalid)
    except ImportError:
        pass

    try:
        import schema
        config.add_view(on_schema_error, context=schema.SchemaError)
    except ImportError as e:
        pass

    try:
        import deform
        config.add_view(on_deform_validation_failure,
                        context=deform.ValidationFailure)
    except ImportError as e:
        pass

    config.add_view(on_error_response, context=ErrorResponse)


__author__ = 'tarzan'

from . import BaseAdapter
import base64
import hashlib
from pyramid.response import Response
import unicodedata
import binascii
import time
import struct

class HttpDigestAdapter(BaseAdapter):

    def _parse_authorization_token(self):
        try:
            if not self.request.authorization:
                return {}
            method, args = self.request.authorization
            if method.lower() != 'digest':
                return {}
            return args
        except (ValueError, TypeError, AttributeError), e:
            return {}

    @property
    def challenge(self):
        try:
            return self._challenge
        except AttributeError:
            self._challenge = self._parse_authorization_token()
        return self._challenge

    @property
    def username(self):
        return self.challenge.get('username', None)

    def unauthenticated_userid(self):
        return self.username

    def _kd_md5(self, msg):
        if isinstance(msg, str):
            msg = msg.encode('utf-8')
        return hashlib.md5(msg).hexdigest()

    def _kd_sha1(self, msg):
        if isinstance(msg, str):
            msg = msg.encode('utf-8')
        return hashlib.sha1(msg).hexdigest()

    def authenticated_userid(self):
        if not self.username:
            return None

        user = self.get_user_callback(self.username)
        password = user['password']
        chals = self.challenge
        realm = chals['realm']
        nonce = chals['nonce']
        qop = chals.get('qop', None)
        uri = chals['uri']
        algorithm = chals.get('algorithm', 'md5').upper()
        opaque = chals.get('opaque')

        if opaque != self._generate_opaque_from_realm(realm):
            return None

        if algorithm == 'MD5':
            hash_func = self._kd_md5
        elif algorithm == 'SHA':
            hash_func = self._kd_sha1
        else:
            assert False, "Algorithm %s has not been implemented" % algorithm

        ha1 = hash_func("%s:%s:%s" % (self.username, realm, password))
        ha2 = hash_func("%s:%s" % (self.request.method.upper(), uri))

        if qop is None:
            response = hash_func("%s:%s:%s" % (ha1, nonce, ha2))
        elif 'auth' in qop.split(','):
            response = hash_func("%s:%s:%s:%s:%s:%s" % (
                ha1,
                nonce,
                chals['nc'],
                chals['cnonce'],
                qop,
                ha2
            ))

        if response != chals['response']:
            return None

        return self.username

    def _generate_realm(self, request):
        return 'API: ' + \
               unicodedata.normalize(
                   'NFKD',
                   unicode(request.path)).encode('ascii','ignore')

    def _generate_nonce(self, request):
        return base64.b64encode(struct.pack('>d', time.time()))

    def _generate_opaque_from_realm(self, realm):
        crc = binascii.crc32(realm, 0x5aa53bb3) & 0xffffffff
        crc = hex(crc)
        if crc[0] == '-':
            crc = '-' + crc[3:]
        else:
            crc = crc[2:]
        return crc

    def login_required(self, request):
        r = Response()
        r.status_code = 401
        r.content_type = 'application/json'
        r.body = '{"errors":{"code":401,"message":"You have to login to access this API"}}'

        realm = self._generate_realm(request)
        r.www_authenticate = ('Digest', {
            'realm': realm,
            'qop': 'auth',
            'algorithm': 'MD5',
            'nonce': self._generate_nonce(request),
            'opaque': self._generate_opaque_from_realm(realm),
        })

        return r


__author__ = 'tarzan'

class BaseAdapter(object):
    def __init__(self, request, get_user_callback):
        self.request = request
        self.get_user_callback = get_user_callback

    def unauthenticated_userid(self):
        return None

    def authenticated_userid(self):
        return None



#coding=utf8
import os
from setuptools import setup, Command

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

requires = [
    'pyramid >= 1.5',
    ]

test_requires = [
    'pytest',
    'pytest-capturelog',
    'webtest',
    'deform',
    'colander',
    'formencode',
    'schema',
]

class PyTest(Command):
    user_options = []
    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import sys,subprocess
        errno = subprocess.call([sys.executable, 'runtests.py'])
        raise SystemExit(errno)


setup(name='pyramid_restful_toolkit',
      version='1.0.3',
      description='Some toolkits for RESTful API development in Pyramid.',
      long_description=README,
      classifiers=[
        "Programming Language :: Python",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        ],
      author='Học .T Đỗ',
      author_email='hoc3010@gmail.com',
      url='https://github.com/tarzanjw/pyramid_restful_toolkit',
      keywords='web wsgi bfg pylons pyramid restful toolkit',
      packages=['pyramid_restful_toolkit', ],
      include_package_data=True,
      zip_safe=False,
      test_suite='pyramid_restful_toolkit',
      install_requires=requires,
      tests_require=requires + test_requires,
      cmdclass = {'test': PyTest},
      entry_points='',
      )



__author__ = 'tarzan'


def is_sqlalchemy_column(col):
    try:
        from sqlalchemy.orm.attributes import InstrumentedAttribute
        from sqlalchemy.orm.properties import ColumnProperty

        return \
            isinstance(col, InstrumentedAttribute) \
            and \
            isinstance(col.property, ColumnProperty)
    except ImportError, e:
        print e
        print col
        return False


def sqlalchemy_obj_to_dict(obj):
    """
    Return a dict from a sqlalchemy object
    :param object obj: object that contains source information
    :return: data from object
    :rtype : dict
    """
    cls = obj.__class__
    data = {a: obj.__getattribute__(a) for a, c in cls.__dict__.items()
            if is_sqlalchemy_column(c)}
    return data